/*
 * MovingAverage.h
 *
 *  Created on: 13.04.2013
 *      Author: nicolas
 */

#ifndef MOVINGAVERAGE_H_
#define MOVINGAVERAGE_H_

#include <stdlib.h>
#include <stdint.h>

template<typename T>
class MovingAverage {
	private:
		T* values;
		size_t maxValues;
		size_t index;
#if ((T) - 1 < 0)
		int32_t sum;
#else
		uint32_t sum;
#endif
		T average;
	public:
		MovingAverage(size_t maxValues, T average = 0) {
			values = (T*) malloc(maxValues * sizeof(T));

			this->maxValues = maxValues;
			index = 0;
			sum = 0;
			this->average = average;
		}
		~MovingAverage() {
			free(values);
		}

		T addValue(T value) {
			sum += value - values[index];
			values[index] = value;
			average = (T) (sum / maxValues);
			if (++index == maxValues) {
				index = 0;
			}
			return average;
		}

		T getAverage() {
			return average;
		}

		void clear(T value) {
			for (size_t i = 0; i < maxValues; i++) {
				values[i] = value;
			}
			average = value;
		}
};

template<>
class MovingAverage<float> {
	private:
		float* values;
		size_t maxValues;
		size_t index;
		uint32_t sum;
		float average;
	public:
		MovingAverage(size_t maxValues, float average = 0.0) {
			values = new float[maxValues];
			this->maxValues = maxValues;
			index = 0;
			sum = 0;
			this->average = average;
		}
		virtual ~MovingAverage() {}

		float addValue(float value) {
			sum -= values[index];
			sum += value;
			values[index] = value;
			average = sum / maxValues;
			if (++index == maxValues) {
				index = 0;
			}
			return average;
		}

		float getAverage() {
			return average;
		}

		void clear(float value) {
			for (size_t i = 0; i < maxValues; i++) {
				values[i] = value;
			}
			average = value;
		}
};

template<>
class MovingAverage<double> {
	private:
		double* values;
		size_t maxValues;
		size_t index;
		uint32_t sum;
		double average;
	public:
		MovingAverage(size_t maxValues, double average = 0.0) {
			values = new double[maxValues];
			this->maxValues = maxValues;
			index = 0;
			sum = 0;
			this->average = average;
		}
		virtual ~MovingAverage() {}

		double addValue(double value) {
			sum -= values[index];
			sum += value;
			values[index] = value;
			average = sum / maxValues;
			if (++index == maxValues) {
				index = 0;
			}
			return average;
		}

		double getAverage() {
			return average;
		}

		void clear(double value) {
			for (size_t i = 0; i < maxValues; i++) {
				values[i] = value;
			}
			average = value;
		}
};

#endif /* MOVINGAVERAGE_H_ */
